package org.processmining.lifecycleperformance.models.performance;

/**
 * Enum that holds the lifecycle performance measures that are stored for the
 * states and transitions of lifecycle models. Currently, uses
 * DescriptiveStatistics for state measurements, which keeps all values in
 * memory, in order to calculate a median. In case this is too heave, change to
 * SummaryStatistics since that does not store all values, only aggregates, but
 * cannot compute median (or other percentiles).
 * 
 * @author B.F.A. Hompes <b.f.a.hompes@tue.nl>
 * 
 */
public enum PerformanceMeasure {
	//@formatter:off
	// State performance measures
	STATE_TIMES_SPENT_SUMMARY("state:times_spent_summary", "times_spent_summary\\(\\s*(\\w*)\\s*\\)"),
	STATE_TIMES_SPENT_MIN("state:times_spent_min", "min\\(times_spent\\(\\s*(\\w*)\\s*\\)\\)"),
	STATE_TIMES_SPENT_MAX("state:times_spent_max","max\\(times_spent\\(\\s*(\\w*)\\s*\\)\\)"),
	STATE_TIMES_SPENT_AVG("state:times_spent_avg","avg\\(times_spent\\(\\s*(\\w*)\\s*\\)\\)"),
	STATE_TIMES_SPENT_MEDIAN("state:times_spent_median","median\\(times_spent\\(\\s*(\\w*)\\s*\\)\\)"),
	STATE_TIMES_SPENT_SUM("state:times_spent_sum","sum\\(times_spent\\(\\s*(\\w*)\\s*\\)\\)"),
	STATE_TIMES_SPENT_COUNT("state:times_spent_count","count\\(times_spent\\(\\s*(\\w*)\\s*\\)\\)"),
//	STATE_TIME_SPENT_COLLECTION("state:times_spent_collection", ""),
	// Transition performance measures
	TRANSITION_TIMES_EXECUTED("transition:timesexecuted","times_executed\\(\\s*(\\w*)\\s*\\)");
//	TRANSITION_COLLECTION("transition:times_executed_collection","");
	
	private String label;
	private String regex;
	
	PerformanceMeasure(String label, String regex) {
		setLabel(label);
		setRegex(regex);
	}
	
	public void setLabel(String label) {
		this.label=label;
	}
	
	public String getLabel() {
		return label;
	}
	
	public void setRegex(String regex) {
		this.regex = regex;
	}
	
	public String getRegex() {
		return regex;
	}
	//@formatter:on
}