package org.processmining.lifecycleperformance.visualization.swing.lifecycle;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.fife.ui.rsyntaxtextarea.RSyntaxTextArea;
import org.fife.ui.rsyntaxtextarea.SyntaxConstants;
import org.fife.ui.rtextarea.RTextScrollPane;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.lifecycleperformance.models.lifecycle.LifecycleModel;
import org.processmining.lifecycleperformance.models.performance.PerformanceMetric;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class PerformanceMetricEditor extends JFrame {

	private static final long serialVersionUID = 7695819649169756983L;

	// FIELDS

	private LifecycleModel lifecycleModel;
	private PerformanceMetric performanceMetric;
	private ProMTextField txtLabel;
	private RSyntaxTextArea txtExpression;
	private ProMTextField txtFormat;

	// CONSTRUCTORS

	public PerformanceMetricEditor(LifecycleModel lifecycleModel, PerformanceMetric performanceMetric) {
		setTitle("Performance metric editor");
		setLifecycleModel(lifecycleModel);
		setPerformanceMetric(performanceMetric);

		txtLabel = new ProMTextField(performanceMetric.getLabel(), "Enter a label for the metric.");

		txtExpression = new RSyntaxTextArea(performanceMetric.getExpression(), 20, 40);
		txtExpression.setSyntaxEditingStyle(SyntaxConstants.SYNTAX_STYLE_GROOVY);
		txtExpression.setCodeFoldingEnabled(true);
		RTextScrollPane spTxtExpression = new RTextScrollPane(txtExpression);

		JButton btnSave = SlickerFactory.instance().createButton("Save");
		btnSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				performanceMetric.setLabel(txtLabel.getText());
				performanceMetric.setExpression(txtExpression.getText());
				performanceMetric.setFormat(txtFormat.getText());
			}
		});

		txtFormat = new ProMTextField(performanceMetric.getFormat(), "Enter a format for the metric.");

		JPanel mainPanel = SlickerFactory.instance().createRoundedPanel();
		mainPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		mainPanel.add(txtLabel, c);
		c.gridy = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		mainPanel.add(txtFormat, c);
		c.gridy = 2;
		c.weighty = 1.0;
		c.fill = GridBagConstraints.BOTH;
		mainPanel.add(spTxtExpression, c);
		c.gridy = 3;
		c.weightx = 0.0;
		c.weighty = 0.0;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.LAST_LINE_END;
		mainPanel.add(btnSave, c);

		setContentPane(mainPanel);

		setSize(600, 400);
		setLocationRelativeTo(null);
		pack();
		setVisible(true);
	}

	// GETTERS AND SETTERS

	public LifecycleModel getLifecycleModel() {
		return lifecycleModel;
	}

	public void setLifecycleModel(LifecycleModel lifecycleModel) {
		this.lifecycleModel = lifecycleModel;
	}

	public PerformanceMetric getPerformanceMetric() {
		return performanceMetric;
	}

	public void setPerformanceMetric(PerformanceMetric performanceMetric) {
		this.performanceMetric = performanceMetric;
	}

}
