package org.processmining.lifecycleperformance.visualization.swing.lifecycle;

import javax.swing.table.DefaultTableModel;

import org.processmining.lifecycleperformance.models.lifecycle.LifecycleModel;
import org.processmining.lifecycleperformance.models.performance.PerformanceMetric;

public class PerformanceMetricsTableModel extends DefaultTableModel {

	private static final long serialVersionUID = 7730514525696626160L;

	// FIELDS

	private LifecycleModel lifecycleModel;

	// CONSTRUCTORS

	public PerformanceMetricsTableModel(LifecycleModel lifecycleModel) {
		super();
		setLifecycleModel(lifecycleModel);
	}

	// GETTERS AND SETTERS

	public LifecycleModel getLifecycleModel() {
		return lifecycleModel;
	}

	public void setLifecycleModel(LifecycleModel lifecycleModel) {
		this.lifecycleModel = lifecycleModel;
	}

	// METHODS

	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}

	@Override
	public int getRowCount() {
		return lifecycleModel == null ? 0
				: lifecycleModel.getPerformanceMetrics() == null ? 0 : lifecycleModel.getPerformanceMetrics().size();
	}

	@Override
	public int getColumnCount() {
		return 3;
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return String.class;
	}

	@Override
	public String getColumnName(int column) {
		switch (column) {
			case 0 :
				return "Metric";
			case 1 :
				return "Format";
			case 2 :
				return "Expression";
		}
		return null;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		PerformanceMetric performanceMetric = lifecycleModel.getPerformanceMetrics().get(rowIndex);
		switch (columnIndex) {
			case 0 :
				return performanceMetric.getLabel();
			case 1 :
				return performanceMetric.getFormat();
			case 2 :
				return performanceMetric.getExpression();
		}
		return null;
	}

	public void add(PerformanceMetric performanceMetric) {
		lifecycleModel.getPerformanceMetrics().add(performanceMetric);
		int row = lifecycleModel.getPerformanceMetrics().indexOf(performanceMetric);
		fireTableRowsInserted(row, row);
		fireTableDataChanged();
	}

	public PerformanceMetric get(int row) {
		if (lifecycleModel.getPerformanceMetrics().size() > row) {
			return lifecycleModel.getPerformanceMetrics().get(row);
		}
		return null;
	}

	public void remove(PerformanceMetric performanceMetric) {
		if (lifecycleModel.getPerformanceMetrics().contains(performanceMetric)) {
			int row = lifecycleModel.getPerformanceMetrics().indexOf(performanceMetric);
			remove(row);
		}
	}

	public void remove(int row) {
		if (lifecycleModel.getPerformanceMetrics().size() > row) {
			lifecycleModel.getPerformanceMetrics().remove(row);
			fireTableRowsDeleted(row, row);
		}
	}

}
